/* -*- pftp-c -*- */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif
#if HAVE_STDIO_H
# include <stdio.h>
#endif
#if HAVE_INTTYPES_H
# include <inttypes.h>
#endif
#if HAVE_STRING_H
# include <string.h>
#endif
#if HAVE_STDLIB_H
# include <stdlib.h>
#endif
#if HAVE_UNISTD_H
# include <unistd.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <pftputil.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "globals.h"
#include "hpanes.h"
#include "site_manager.h"
#include "about_dlg.h"

#ifdef WITH_DMALLOC
# include <dmalloc.h>
#endif

void
on_main_window_destroy                 (GtkObject       *object,
                                        gpointer         user_data)
{
    gtk_main_quit();
}


void
on_newsite_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_load_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_save_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_save_as_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_reconnect_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    if (curlist && getListViewName(curlist) && !lockedListView(curlist)) {
	char *name = (char *)strdup(getListViewName(curlist));
	setListViewFTP(curlist, NULL);
	setListViewFTP(curlist, name);
	free(name);
    }
}


void
on_close_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    if (curlist && !lockedListView(curlist)) 
	setListViewFTP(curlist, NULL);
}


void
on_quit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    gtk_main_quit();
}


void
on_site_manager_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    showSiteManager();
}


void
on_enqueue_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    /* Allowed even if listview is locked. */
    if (curlist) {
	list_view_file_t *item = NULL;
	size_t count = selectedListView(curlist, &item);
	
	enqueueView(queue, curlist, curlist == left ? right : left,
		    count, item);
	
	freeSelectedListView(&item, count);
    }
}


void
on_dequeue_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    dequeueView(queue);
}


void
on_begin_transfer_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    startQueueView(queue);
}


void
on_abort_transfer_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    abortQueueView(queue);
}


void
on_pause_queue_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    pauseQueueView(queue);
}


void
on_site_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    if (curlist && getListViewName(curlist) && !lockedListView(curlist))
	run_rawcmd(1);
}


void
on_raw_command_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    if (curlist && getListViewName(curlist) && !lockedListView(curlist))
	run_rawcmd(0);
}


void
on_refresh_filelist_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    if (curlist && !lockedListView(curlist))
	refreshListView(curlist);
}


void
on_index_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_about_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    about_dlg();
}

void
loadmenuitem_activate                  (GtkMenuItem     *menuitem,
					gpointer         user_data)
{
    size_t id = (size_t) user_data;
    pftp_settings_t settings = pftp_getFTPSettings2(id);

    if (settings && curlist && !lockedListView(curlist)) {
	setListViewFTP(curlist, settings->name);
    }
}

void
on_new_button_clicked                  (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{

}


void
on_save_button_clicked                 (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{

}


void
on_close_button_clicked                (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
    if (curlist && !lockedListView(curlist)) 
	setListViewFTP(curlist, NULL);
}


void
on_site_manager_button_clicked         (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
    showSiteManager();
}


void
on_enqueue_button_clicked              (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
    on_enqueue_activate(NULL, NULL);
}


void
on_dequeue_button_clicked              (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{

}


void
on_begin_transfer_button_clicked       (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
    on_begin_transfer_activate(NULL, NULL);
}


void
on_abort_transfer_button_clicked       (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
    on_abort_transfer_activate(NULL, NULL);
}


void
on_pause_queue_button_clicked          (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
    on_pause_queue_activate(NULL, NULL);
}


void
on_refresh_button_clicked              (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
    if (curlist && !lockedListView(curlist))
	refreshListView(curlist);
}

gboolean
on_view_label2_box_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
    selectListView(left, 0);
    selectListView(right, 1);
    return FALSE;
}

gboolean
on_view_label1_box_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
    selectListView(left, 1);
    selectListView(right, 0);
    return FALSE;
}




void
on_filelist1_row_activated             (GtkTreeView     *treeview,
                                        GtkTreePath     *path,
                                        GtkTreeViewColumn *column,
                                        gpointer         user_data)
{
    if (left && !lockedListView(left))
	actionRowListView(left, path);
}


void
on_filelist2_row_activated             (GtkTreeView     *treeview,
                                        GtkTreePath     *path,
                                        GtkTreeViewColumn *column,
                                        gpointer         user_data)
{
    if (right && !lockedListView(right))
	actionRowListView(right, path);
}


void
on_main_window_size_allocate           (GtkWidget       *widget,
                                        GdkRectangle    *allocation,
                                        gpointer         user_data)
{
    if (global_config) {
	global_config->gui_width = allocation->width;
	global_config->gui_height = allocation->height;
    }
    updateHPanes(widget, allocation);
}

gboolean
on_filelist1_focus_in_event            (GtkWidget       *widget,
                                        GdkEventFocus   *event,
                                        gpointer         user_data)
{
    selectListView(left, 1);
    selectListView(right, 0);
    return FALSE;
}


gboolean
on_filelist2_focus_in_event            (GtkWidget       *widget,
                                        GdkEventFocus   *event,
                                        gpointer         user_data)
{
    selectListView(left, 0);
    selectListView(right, 1);
    return FALSE;
}


void
on_make_new_directory_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    if (curlist && !lockedListView(curlist))
	make_new_dir();
}


void
on_rename_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    if (curlist && !lockedListView(curlist))
	rename_item();
}


void
on_delete_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    if (curlist && !lockedListView(curlist))
	delete_item();
}


void
on_directory1_changed                  (GtkComboBox     *combobox,
                                        gpointer         user_data)
{
    if (left && !lockedListView(left))
	changeDirectoryListView(left);
}


void
on_directory2_changed                  (GtkComboBox     *combobox,
                                        gpointer         user_data)
{
    if (right && !lockedListView(right))
	changeDirectoryListView(right);
}

static gboolean filelist_key_release_event(list_view_t list,
					   GtkWidget       *widget,
					   GdkEventKey     *event,
					   gpointer         user_data)
{
    if (event->keyval == GDK_BackSpace && event->state == 0) {
	const char *ftpname;
	if (!list || lockedListView(list))
	    return FALSE;
	ftpname = getListViewName(list);
	if (ftpname) {
	    pftp_server_t ftp = getListViewFTP(list, 0);
	    if (ftp)
		pftp_cd(ftp, "..", NULL);
	    updateListViewDir(list);
	} else {
	    setListViewLocalPath(list, "..");
	}
    }
    
    return FALSE;
}

gboolean
on_filelist1_key_release_event         (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
    return filelist_key_release_event(left, widget, event, user_data);
}


gboolean
on_filelist2_key_release_event         (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
    return filelist_key_release_event(right, widget, event, user_data);
}


void
on_add_host_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
    siteManagerAddHost();
}


void
on_remove_host_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
    siteManagerDelHost();
}


void
on_use_secure_toggled                  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    siteManagerUseSecure();
}


void
on_add_prio_mask_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
    siteManagerAddPrioMask();
}


void
on_edit_prio_mask_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
    siteManagerEditPrioMask();
}


void
on_del_prio_mask_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
    siteManagerDelPrioMask();
}


void
on_add_skip_mask_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
    siteManagerAddSkipMask();
}


void
on_edit_skip_mask_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
    siteManagerEditSkipMask();
}


void
on_del_skip_mask_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
    siteManagerDelSkipMask();
}

void
on_edit_host_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
    siteManagerEditHost();
}


void
on_new_site_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
    siteManagerNewSite();
}


void
on_delete_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
    
}


void
on_connect_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_close_clicked                       (GtkButton       *button,
                                        gpointer         user_data)
{
    closeSiteManager();
}


void
on_site_tree_cursor_changed            (GtkTreeView     *treeview,
                                        gpointer         user_data)
{
    siteManagerSiteTreeSelection();
}


void
on_host_list_cursor_changed            (GtkTreeView     *treeview,
                                        gpointer         user_data)
{

}


void
on_priority_list_cursor_changed        (GtkTreeView     *treeview,
                                        gpointer         user_data)
{
    siteManagerPrioListSelection();
}


void
on_ignore_list_cursor_changed          (GtkTreeView     *treeview,
                                        gpointer         user_data)
{
    siteManagerIgnoreListSelection();
}


void
on_move_up_prio_mask_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
    siteManagerMoveUpPrioMask();
}


void
on_move_down_prio_mask_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
    siteManagerMoveDownPrioMask();
}

