/* -*- pftp-c -*- */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif
#if HAVE_STDIO_H
# include <stdio.h>
#endif
#if HAVE_STDLIB_H
# include <stdlib.h>
#endif
#if HAVE_STRING_H
# include <string.h>
#endif
#if HAVE_UNISTD_H
# include <unistd.h>
#endif
#if HAVE_INTTYPES_H
# include <inttypes.h>
#endif
#if HAVE_ERRNO_H
# include <errno.h>
#endif
#if HAVE_TIME_H
# include <time.h>
#endif

#include <gtk/gtk.h>
#include <str.h>
#include <pftputil.h>
#include <pftp_client.h>

#include "support.h"
#include "rename_dlg.h"
#include "messagebox.h"
#include "globals.h"
#include "utf8.h"

#ifdef WITH_DMALLOC
# include <dmalloc.h>
#endif

void delete_item()
{
    list_view_file_t *item = NULL;
    size_t c, count = 0;
    pftp_server_t ftp = NULL;
    GtkResponseType ret;
    
    if (curlist) {
	ftp = getListViewFTP(curlist, 0);
	if (!ftp && getListViewName(curlist))
	    return;
	count = selectedListView(curlist, &item);
	if (count == 0) {
	    freeSelectedListView(&item, count);
	    return;
	}
    } else
	return;
        
    if (count == 1) {
	char *tmp;
	tmp = strdup(item[0].filename);
	make_buffer_utf8_safe(&tmp, NULL);
	ret = messageBox(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION,
			 GTK_BUTTONS_YES_NO, "Really delete `%s'?", tmp);
	free(tmp);
    } else {
	ret = messageBox(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION,
			 GTK_BUTTONS_YES_NO, "Really delete all %z items?", 
			 count);
    }

    if (ret == GTK_RESPONSE_YES) {
	char *path = NULL;
	if (!pftp_get_path_dir(getListViewName(curlist), &path)) {
	    pftp_que_t rmque, rmerrque;

	    rmque = pftp_CreateQue();
	    rmerrque = pftp_CreateQue();

	    for (c = 0; c < count; c++) {
		pftp_item_t qitem;
		qitem = malloc(sizeof(struct pftp_item_s));
		memset(qitem, 0, sizeof(struct pftp_item_s));
		qitem->ftp_from = getListViewName(curlist) 
		    ? strdup(getListViewName(curlist)) : NULL;
		qitem->dir = item[c].directory;
		qitem->srcpath = strdup(path);
		qitem->filename = strdup(item[c].filename);
		qitem->action = PFTP_DELETE_FROM;
		qitem->preserve = PFTP_PRESERVE_NONE;
		qitem->mode = PFTP_MODE_NORMAL;
		pftp_que_add_last(qitem, rmque);         
	    }

	    if (pftp_queLength(rmque) > 0) {
		pftp_begin_trans(NULL, rmque, rmerrque);
	    }

	    pftp_clear_que(rmque);
	    pftp_clear_que(rmerrque);
	    pftp_FreeQue(rmque);
	    pftp_FreeQue(rmerrque);                    
	}

	if (path) free(path);
    }

    freeSelectedListView(&item, count);
}

