/* -*- pftp-c -*- */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif
#if HAVE_STDIO_H
# include <stdio.h>
#endif
#if HAVE_STDLIB_H
# include <stdlib.h>
#endif
#if HAVE_STRING_H
# include <string.h>
#endif
#if HAVE_UNISTD_H
# include <unistd.h>
#endif
#if HAVE_INTTYPES_H
# include <inttypes.h>
#endif

#include <gtk/gtk.h>
#include <str.h>

#include "support.h"
#include "messagebox.h"
#include "globals.h"

#ifdef WITH_DMALLOC
# include <dmalloc.h>
#endif

// From main.c
extern GtkWidget *main_window;

GtkResponseType messageBox(GtkWindow *parent, GtkDialogFlags flags,
			   GtkMessageType type, GtkButtonsType buttons,
			   const char *format, ...)
{
    GtkResponseType ret;
    GtkWidget *dlg;
    va_list list;
#if !HAVE_VASPRINTF
    size_t newformatlen;
#endif
    char *newformat;

    va_start(list, format);

#if HAVE_VASPRINTF
    newformat = NULL;
    vasprintf(&newformat, format, list);
#else
    newformatlen = 1024 + strlen(format);
    newformat = malloc(newformatlen + 1);

    vsnprintf(newformat, newformatlen, format, list);
    newformat[newformatlen] = '\0';
#endif

    if (parent == NULL)
	parent = GTK_WINDOW(main_window);
        
    dlg = gtk_message_dialog_new(parent, flags, type, buttons, newformat);
    ret = gtk_dialog_run(GTK_DIALOG(dlg));
    gtk_widget_destroy(dlg);

    free(newformat);
    va_end(list);
    
    return ret;
}

