/* -*- pftp-c -*- */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif
#if HAVE_STDIO_H
# include <stdio.h>
#endif
#if HAVE_STDLIB_H
# include <stdlib.h>
#endif
#if HAVE_STRING_H
# include <string.h>
#endif

#include <gtk/gtk.h>

#include "support.h"
#include "need_pass_dlg.h"
#include "utf8.h"

#ifdef WITH_DMALLOC
# include <dmalloc.h>
#endif

struct need_pass_dlg_s {
    GtkDialog *dlg;
    GtkEntry *username, *password;
    GtkLabel *u_label, *p_label;
};

need_pass_dlg_t initNeedPassDlg(GtkWidget *dlg, GtkWidget *parent)
{
    need_pass_dlg_t ret = malloc(sizeof(struct need_pass_dlg_s));
    memset(ret, 0, sizeof(struct need_pass_dlg_s));
    
    ret->dlg = GTK_DIALOG(dlg);
    
    gtk_window_set_transient_for(GTK_WINDOW(ret->dlg), GTK_WINDOW(parent));
    gtk_dialog_set_default_response(ret->dlg, GTK_RESPONSE_OK);
    
    ret->username = GTK_ENTRY(lookup_widget(dlg, "username"));
    ret->u_label = GTK_LABEL(lookup_widget(dlg, "username_label"));
    ret->password = GTK_ENTRY(lookup_widget(dlg, "password"));
    ret->p_label = GTK_LABEL(lookup_widget(dlg, "password_label"));
    
    return ret;
}

void freeNeedPassDlg(need_pass_dlg_t dlg)
{
    if (dlg) {
	free(dlg);
    }
}

int doNeedPassDlg(need_pass_dlg_t dlg, char **username, char **password)
{
    int ret;
    
    if (username) {
	gtk_widget_show(GTK_WIDGET(dlg->u_label));
	gtk_widget_show(GTK_WIDGET(dlg->username));
	
	if (*username) {	
	    make_buffer_utf8_safe(username, NULL);
	    gtk_entry_set_text(dlg->username, (*username));
	} else {
	    gtk_entry_set_text(dlg->username, "");
	}
    } else {
	gtk_widget_hide(GTK_WIDGET(dlg->u_label));
	gtk_widget_hide(GTK_WIDGET(dlg->username));
    }
    if (password) {
	gtk_widget_show(GTK_WIDGET(dlg->p_label));
	gtk_widget_show(GTK_WIDGET(dlg->password));
	
	gtk_entry_set_text(dlg->password, "");
    } else {
	gtk_widget_hide(GTK_WIDGET(dlg->p_label));
	gtk_widget_hide(GTK_WIDGET(dlg->password));
    }
    
    ret = gtk_dialog_run(dlg->dlg);
    
    if (ret == GTK_RESPONSE_OK) {
	if (username) {
	    if ((*username)) free((*username));
	    (*username) = strdup(gtk_entry_get_text(dlg->username));
	    make_ascii_again(*username, NULL);
	}
	if (password) {
	    if ((*password)) free((*password));
	    (*password) = strdup(gtk_entry_get_text(dlg->password));
	    make_ascii_again(*password, NULL);
	}
    }
    
    gtk_widget_hide(GTK_WIDGET(dlg->dlg));
    
    if (ret != GTK_RESPONSE_OK) {
	return 0;    
    }
    
    return 1;
}

