/* -*- pftp-c -*- */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif
#if HAVE_STDARG_H
# include <stdarg.h>
#endif
#if HAVE_STDLIB_H
# include <stdlib.h>
#endif
#if HAVE_STRING_H
# include <string.h>
#endif
#if HAVE_STDIO_H
# include <stdio.h>
#endif

#include <gtk/gtk.h>
#include <str.h>

#include "output_view.h"
#include "utf8.h"

#ifdef WITH_DMALLOC
# include <dmalloc.h>
#endif

struct output_view_s
{
    GtkTextView *view;
    GtkTextBuffer *buffer;
    GtkTextTag **tag;
};

output_view_t initOutputView(GtkTextView *view)
{
    output_view_t ret;

    ret = (output_view_t) malloc(sizeof(struct output_view_s));
    memset(ret, 0, sizeof(struct output_view_s));
    
    ret->view = view;
    ret->buffer = gtk_text_view_get_buffer(ret->view);
    gtk_text_buffer_set_text(ret->buffer, "", 0);       

    ret->tag = (GtkTextTag **) malloc(N_TYPES * sizeof(GtkTextTag *));
    ret->tag[STATUS] = gtk_text_buffer_create_tag(ret->buffer, "status_tag",
						  "foreground", "black", 
						  "family", "Monospace",
						  "family-set", TRUE,
						  NULL);
    ret->tag[ERR] = gtk_text_buffer_create_tag(ret->buffer, "error_tag",
					       "foreground", "red", 
					       "family", "Monospace",
					       "family-set", TRUE,
					       NULL);
    ret->tag[LIST1] = gtk_text_buffer_create_tag(ret->buffer, "list1_tag",
						 "foreground", "darkgreen", 
						 "family", "Monospace",
						 "family-set", TRUE,
						 NULL);
    ret->tag[LIST2] = gtk_text_buffer_create_tag(ret->buffer, "list2_tag",
						 "foreground", "darkblue", 
						 "family", "Monospace",
						 "family-set", TRUE,
						 NULL);

    return ret;
}

void freeOutputView(output_view_t view)
{
    if (view) {
	if (view->tag)
	    free(view->tag);
	free(view);    
    }
}

inline static int is_digit(char c)
{
    return (c >= '0' && c <= '9');
}

void output_printf(output_view_t view, output_type_t type, 
		   const char *format, ...)
{
    va_list list;
    char *buffer = NULL;
    GtkTextIter start, end;
//    GtkAdjustment *adjust;
    GtkTextMark *endmark;
    size_t len;
    gint chars;

    va_start(list, format);
    
#if HAVE_VASPRINTF
    vasprintf(&buffer, format, list);
#else
    len = strlen(format) + 500;
    buffer = (char *) malloc(len);
    
    vsnprintf(buffer, len, format, list);
#endif
    
    len = strlen(buffer);
    
    if (type == LIST1 || type == LIST2) {
	if (len > 6 && is_digit(buffer[3]) && is_digit(buffer[4]) && 
	    is_digit(buffer[5]) && is_whitespace(buffer[6]) && 
	    strchr(buffer + 6, '\n')) {
	    buffer[6] = '\n';
	}
    }

    make_buffer_utf8_safe(&buffer, &len);
    
    chars = gtk_text_buffer_get_char_count(view->buffer);
    gtk_text_buffer_get_end_iter(view->buffer, &start);
    gtk_text_buffer_insert(view->buffer, &start, buffer, len);
    gtk_text_buffer_get_end_iter(view->buffer, &end);
    gtk_text_buffer_get_iter_at_offset(view->buffer, &start, chars);
    gtk_text_buffer_apply_tag(view->buffer, view->tag[type], &start, &end);
    gtk_text_buffer_place_cursor(view->buffer, &end);
    
    endmark = gtk_text_buffer_create_mark(view->buffer, "endmark", &end, 
					  FALSE);
    gtk_text_view_scroll_to_mark(view->view, endmark, 0.0, TRUE, 0.0, 1.0);
    gtk_text_buffer_delete_mark(view->buffer, endmark);
    
    free(buffer);
    
    va_end(list);
}
