/* -*- pftp-c -*- */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif
#if HAVE_STDIO_H
# include <stdio.h>
#endif
#if HAVE_STDLIB_H
# include <stdlib.h>
#endif
#if HAVE_STRING_H
# include <string.h>
#endif
#if HAVE_UNISTD_H
# include <unistd.h>
#endif
#if HAVE_INTTYPES_H
# include <inttypes.h>
#endif
#if HAVE_ERRNO_H
# include <errno.h>
#endif

#include <gtk/gtk.h>
#include <pftp.h>
#include <str.h>

#include "support.h"
#include "rename_dlg.h"
#include "messagebox.h"
#include "globals.h"
#include "utf8.h"

#ifdef WITH_DMALLOC
# include <dmalloc.h>
#endif

// From main.c
extern rename_dlg_t renamedlg;

struct rename_dlg_s {
    GtkDialog *dlg;
    GtkEntry *newname;
};

rename_dlg_t initRenameDlg(GtkWidget *dlg, GtkWidget *parent)
{
    rename_dlg_t ret = malloc(sizeof(struct rename_dlg_s));
    memset(ret, 0, sizeof(struct rename_dlg_s));
    
    ret->dlg = GTK_DIALOG(dlg);
    
    gtk_window_set_transient_for(GTK_WINDOW(ret->dlg), GTK_WINDOW(parent));
    gtk_dialog_set_default_response(ret->dlg, GTK_RESPONSE_OK);
    
    ret->newname = GTK_ENTRY(lookup_widget(dlg, "new_name"));
    
    return ret;
}

void freeRenameDlg(rename_dlg_t dlg)
{
    if (dlg) {
	free(dlg);
    }
}

static int doRenameDlg(rename_dlg_t dlg, const char *oldname, char **newname)
{
    int ret;
    char *tmp;

    tmp = strdup(oldname);
    make_buffer_utf8_safe(&tmp, NULL);
    gtk_entry_set_text(dlg->newname, tmp);
    free(tmp);
    gtk_editable_select_region(GTK_EDITABLE(dlg->newname), 0, -1);

    ret = gtk_dialog_run(dlg->dlg);
    
    if ((*newname)) free((*newname));
    
    if (ret == GTK_RESPONSE_OK) {
	(*newname) = strdup(gtk_entry_get_text(dlg->newname));
	make_ascii_again((*newname), NULL);
    }
    
    gtk_widget_hide(GTK_WIDGET(dlg->dlg));
    
    if (ret != GTK_RESPONSE_OK) {
	return 0;    
    }
    
    return 1;
}

void rename_item()
{
    char *newname = NULL;
    list_view_file_t *item = NULL;
    pftp_server_t ftp = NULL;
    size_t count = 0;
    
    if (curlist) {
	ftp = getListViewFTP(curlist, 0);
	if (!ftp && getListViewName(curlist))
	    return;
	count = selectedListView(curlist, &item);
	if (count == 0) {
	    freeSelectedListView(&item, count);
	    return;
	}
	if (count > 1) {
	    messageBox(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
		       GTK_BUTTONS_OK, "Can only rename one item at a time.");
	    freeSelectedListView(&item, count);
	    return;
	}
    } else
	return;
    
    if (doRenameDlg(renamedlg, item[0].filename, &newname)) {
	trim(newname);
	if (newname[0] && strcmp(item[0].filename, newname)) {
	    if (ftp) {
		if (!pftp_rename(ftp, item[0].filename, newname)) {
		    pftp_clear_dir_cache(ftp, ".");
		}

		updateListViewDir(curlist);
	    } else {
		if (rename(item[0].filename, newname)) {
		    messageBox(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
			       GTK_BUTTONS_OK, "Error: %s", strerror(errno));
		}
	    }	   
	}
    }
    
    if (newname) free(newname);
    freeSelectedListView(&item, count);
}
