/* -*- pftp-c -*- */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif
#if HAVE_STDIO_H
# include <stdio.h>
#endif
#if HAVE_STDLIB_H
# include <stdlib.h>
#endif
#if HAVE_STRING_H
# include <string.h>
#endif
#if HAVE_UNISTD_H
# include <unistd.h>
#endif
#if HAVE_INTTYPES_H
# include <inttypes.h>
#endif
#if HAVE_ERRNO_H
# include <errno.h>
#endif

#include <gtk/gtk.h>
#include <pftp.h>
#include <str.h>

#include "support.h"
#include "site_manager.h"
#include "messagebox.h"
#include "globals.h"

#ifdef WITH_DMALLOC
# include <dmalloc.h>
#endif

// From main.c
extern site_manager_t site_manager;

struct site_manager_s {
    GtkWindow *wnd;
    GtkTreeView *site_tree;
    GtkButton *new_site, *delete, *connect, *close;
    
    // general
    GtkEntry *site_name, *username, *password, *remote_path, *local_path;
    GtkTreeView *host_list;
    GtkButton *add_host, *edit_host, *remove_host;

    GtkListStore *site_list, *host_store;

    // options
    GtkCheckButton *use_passive, *cache_dirs;

    // advanced
    GtkEntry *account, *port_start, *port_end, *bind_to, *myaddr;

    // secure
    GtkCheckButton *use_secure, *secure_data, *secure_list;
    GtkRadioButton *implicid_ssl, *auth_tls;

    // prioritylist
    GtkTreeView *priority_list;
    GtkEntry *prio_mask;
    GtkButton *add_prio_mask, *edit_prio_mask, *del_prio_mask;
    GtkCheckButton *include_global_prio;

    GtkListStore *prio_list;

    // ignorelist
    GtkTreeView *ignore_list;
    GtkEntry *skip_mask;
    GtkButton *add_skip_mask, *edit_skip_mask, *del_skip_mask;
    GtkCheckButton *include_global_skip;    

    GtkListStore *skip_list;
};

static void fillSiteList(site_manager_t wnd);
static void loadSiteTabs(site_manager_t wnd, GtkTreeIter *iter);
static void setCheckButton(GtkCheckButton *btn, int val);

site_manager_t initSiteManager(GtkWidget *wnd, GtkWidget *parent)
{
    GtkCellRenderer *renderer;
    GtkTreeViewColumn *column;
    site_manager_t ret = malloc(sizeof(struct site_manager_s));
    memset(ret, 0, sizeof(struct site_manager_s));

    renderer = gtk_cell_renderer_text_new();    

    ret->wnd = GTK_WINDOW(wnd);
    
    gtk_window_set_transient_for(ret->wnd, GTK_WINDOW(parent));
    
    ret->site_tree = GTK_TREE_VIEW(lookup_widget(wnd, "site_tree"));
    ret->new_site = GTK_BUTTON(lookup_widget(wnd, "new_site"));
    ret->delete = GTK_BUTTON(lookup_widget(wnd, "delete"));
    ret->connect = GTK_BUTTON(lookup_widget(wnd, "connect"));
    ret->close = GTK_BUTTON(lookup_widget(wnd, "close"));

    // general
    ret->site_name = GTK_ENTRY(lookup_widget(wnd, "site_name"));
    ret->username = GTK_ENTRY(lookup_widget(wnd, "username"));
    ret->password = GTK_ENTRY(lookup_widget(wnd, "password"));
    ret->remote_path = GTK_ENTRY(lookup_widget(wnd, "remote_path"));
    ret->local_path = GTK_ENTRY(lookup_widget(wnd, "local_path"));
    ret->host_list = GTK_TREE_VIEW(lookup_widget(wnd, "host_list"));
    ret->add_host = GTK_BUTTON(lookup_widget(wnd, "add_host"));
    ret->edit_host = GTK_BUTTON(lookup_widget(wnd, "edit_host"));
    ret->remove_host = GTK_BUTTON(lookup_widget(wnd, "remove_host"));

    ret->site_list = gtk_list_store_new(1, G_TYPE_STRING);
    gtk_tree_view_set_model(ret->site_tree, GTK_TREE_MODEL(ret->site_list));
    column = gtk_tree_view_column_new_with_attributes("", renderer, "text", 
						      0, NULL);
    gtk_tree_view_column_set_resizable(column, TRUE);
    gtk_tree_view_append_column(ret->site_tree, column);

    ret->host_store = gtk_list_store_new(1, G_TYPE_STRING);
    gtk_tree_view_set_model(ret->host_list, GTK_TREE_MODEL(ret->host_store));
    column = gtk_tree_view_column_new_with_attributes("", renderer, "text", 
						      0, NULL);
    gtk_tree_view_column_set_resizable(column, TRUE);
    gtk_tree_view_append_column(ret->host_list, column);

    // options
    ret->use_passive = GTK_CHECK_BUTTON(lookup_widget(wnd, "use_passive"));
    ret->cache_dirs = GTK_CHECK_BUTTON(lookup_widget(wnd, "cache_dirs"));

    // advanced
    ret->account  = GTK_ENTRY(lookup_widget(wnd, "account"));
    ret->port_start = GTK_ENTRY(lookup_widget(wnd, "port_start"));
    ret->port_end = GTK_ENTRY(lookup_widget(wnd, "port_end"));
    ret->bind_to = GTK_ENTRY(lookup_widget(wnd, "bind_to"));
    ret->myaddr = GTK_ENTRY(lookup_widget(wnd, "myaddr"));

    // secure
    ret->use_secure = GTK_CHECK_BUTTON(lookup_widget(wnd, "use_secure"));
    ret->secure_data = GTK_CHECK_BUTTON(lookup_widget(wnd, "secure_data"));
    ret->secure_list = GTK_CHECK_BUTTON(lookup_widget(wnd, "secure_list"));
    ret->implicid_ssl = GTK_RADIO_BUTTON(lookup_widget(wnd, "implicid_ssl"));
    ret->auth_tls = GTK_RADIO_BUTTON(lookup_widget(wnd, "auth_tls"));

    // prioritylist
    ret->priority_list = GTK_TREE_VIEW(lookup_widget(wnd, "priority_list"));
    ret->prio_mask = GTK_ENTRY(lookup_widget(wnd, "prio_mask"));
    ret->add_prio_mask = GTK_BUTTON(lookup_widget(wnd, "add_prio_mask"));
    ret->edit_prio_mask = GTK_BUTTON(lookup_widget(wnd, "edit_prio_mask"));
    ret->del_prio_mask = GTK_BUTTON(lookup_widget(wnd, "del_prio_mask"));
    ret->include_global_prio = 
	GTK_CHECK_BUTTON(lookup_widget(wnd, "include_global_prio"));

    ret->prio_list = gtk_list_store_new(1, G_TYPE_STRING);
    gtk_tree_view_set_model(ret->priority_list, 
			    GTK_TREE_MODEL(ret->prio_list));
    column = gtk_tree_view_column_new_with_attributes("", renderer, "text", 
						      0, NULL);
    gtk_tree_view_column_set_resizable(column, TRUE);
    gtk_tree_view_append_column(ret->priority_list, column);

    // ignorelist
    ret->ignore_list = GTK_TREE_VIEW(lookup_widget(wnd, "ignore_list"));
    ret->skip_mask = GTK_ENTRY(lookup_widget(wnd, "skip_mask"));
    ret->add_skip_mask = GTK_BUTTON(lookup_widget(wnd, "add_skip_mask"));
    ret->edit_skip_mask = GTK_BUTTON(lookup_widget(wnd, "edit_skip_mask"));
    ret->del_skip_mask = GTK_BUTTON(lookup_widget(wnd, "del_skip_mask"));
    ret->include_global_skip = 
	GTK_CHECK_BUTTON(lookup_widget(wnd, "include_global_skip"));
    
    ret->skip_list = gtk_list_store_new(1, G_TYPE_STRING);
    gtk_tree_view_set_model(ret->ignore_list, 
			    GTK_TREE_MODEL(ret->skip_list));
    column = gtk_tree_view_column_new_with_attributes("", renderer, "text", 
						      0, NULL);
    gtk_tree_view_column_set_resizable(column, TRUE);
    gtk_tree_view_append_column(ret->ignore_list, column);
    
    return ret;
}

void freeSiteManager(site_manager_t wnd)
{
    if (wnd) {
	free(wnd);
    }
}

void showSiteManager(void)
{
    fillSiteList(site_manager);
    gtk_widget_show(GTK_WIDGET(site_manager->wnd));
}

void closeSiteManager(void)
{
    gtk_widget_hide(GTK_WIDGET(site_manager->wnd));
}

void fillSiteList(site_manager_t wnd)
{
    GtkTreeIter iter;
    GtkTreeSelection *sel = gtk_tree_view_get_selection(wnd->site_tree);
    size_t s;
    int first = 1;

    gtk_list_store_clear(wnd->site_list);

    for (s = 0; s < pftp_getNrOfSettings(); s++) {
	pftp_settings_t settings = pftp_getFTPSettings2(s);	
	if (settings) {
	    gtk_list_store_append(wnd->site_list, &iter);
	    gtk_list_store_set(wnd->site_list, &iter, 0, 
			       settings->name, -1);
	    if (first) {
		gtk_tree_selection_select_iter(sel, &iter);
		first = 0;
	    }
	}
    }

    gtk_tree_view_columns_autosize(wnd->site_tree);
}

void siteManagerSiteTreeSelection(void)
{
    GtkTreeSelection *sel;	
    GtkTreeIter iter;
    sel = gtk_tree_view_get_selection(site_manager->site_tree);
    if (gtk_tree_selection_get_selected(sel, NULL, &iter)) {
	loadSiteTabs(site_manager, &iter);
    }
}

static void loadGeneralTab(site_manager_t wnd, pftp_settings_t settings);
static void loadOptionsTab(site_manager_t wnd, pftp_settings_t settings);
static void loadAdvancedTab(site_manager_t wnd, pftp_settings_t settings);
static void loadSecureTab(site_manager_t wnd, pftp_settings_t settings);
static void loadPrioTab(site_manager_t wnd, pftp_settings_t settings);
static void loadSkipTab(site_manager_t wnd, pftp_settings_t settings);

void loadSiteTabs(site_manager_t wnd, GtkTreeIter *iter)
{
    GValue value;
    pftp_settings_t settings;
    memset(&value, 0, sizeof(GValue));
    gtk_tree_model_get_value(GTK_TREE_MODEL(wnd->site_list), iter, 0, &value);
    settings = pftp_getFTPSettings(g_value_get_string(&value));
    g_value_unset(&value);

    loadGeneralTab(wnd, settings);
    loadOptionsTab(wnd, settings);
    loadAdvancedTab(wnd, settings);
    loadSecureTab(wnd, settings);
    loadPrioTab(wnd, settings);
    loadSkipTab(wnd, settings);
}

void loadGeneralTab(site_manager_t wnd, pftp_settings_t settings)
{
    int first = 1;
    size_t h, h_len;
    char *tmp = NULL;
    GtkTreeSelection *sel = gtk_tree_view_get_selection(wnd->host_list);
    GtkTreeIter iter;

    gtk_entry_set_text(wnd->site_name, settings->name);
    
    gtk_list_store_clear(wnd->host_store);
    
    for (h = 0; h < settings->hosts; h++) {
	h_len = strlen(settings->hostname[h]);
	tmp = realloc(tmp, h_len + 10);
	memcpy(tmp, settings->hostname[h], h_len);
	tmp[h_len] = ':';
	snprintf(tmp + h_len + 1, 9, "%u", settings->port[h]);
	gtk_list_store_append(wnd->host_store, &iter);
	gtk_list_store_set(wnd->host_store, &iter, 0, tmp, -1);
	if (first) {
	    gtk_tree_selection_select_iter(sel, &iter);
	    first = 0;
	}
    }
    
    if (tmp) free(tmp);
    
    gtk_tree_view_columns_autosize(wnd->host_list);

    gtk_entry_set_text(wnd->username, settings->username ? 
		       settings->username : "");
    gtk_entry_set_text(wnd->password, settings->password ?
		       settings->password : "");
    gtk_entry_set_text(wnd->remote_path, ""); // !!!
    gtk_widget_set_sensitive(GTK_WIDGET(wnd->remote_path), FALSE);
    gtk_entry_set_text(wnd->local_path, ""); // !!!
    gtk_widget_set_sensitive(GTK_WIDGET(wnd->local_path), FALSE);
}

void loadOptionsTab(site_manager_t wnd, pftp_settings_t settings)
{
    setCheckButton(wnd->use_passive, settings->pasv);
    setCheckButton(wnd->cache_dirs, -1); // !!!
    gtk_widget_set_sensitive(GTK_WIDGET(wnd->cache_dirs), FALSE);
}

void loadAdvancedTab(site_manager_t wnd, pftp_settings_t settings)
{
    char tmp[10];
    uint16_t port;

    gtk_entry_set_text(wnd->account, settings->account ?
		       settings->account : "");

    port = settings->port_start ? settings->port_start :
	settings->global->port_start;
    
    if (port) {
	snprintf(tmp, 10, "%u", port);
	gtk_entry_set_text(wnd->port_start, tmp);
    } else {
	gtk_entry_set_text(wnd->port_start, "");
    }

    port = settings->port_stop ? settings->port_stop :
	settings->global->port_stop;

    if (port) {
	snprintf(tmp, 10, "%u", port);
	gtk_entry_set_text(wnd->port_end, tmp);
    } else {
	gtk_entry_set_text(wnd->port_end, "");
    }
    
    if (settings->bind_to)
	gtk_entry_set_text(wnd->bind_to, settings->bind_to);
    else
	gtk_entry_set_text(wnd->bind_to, settings->global->bind_to ?
			   settings->global->bind_to : "");
    if (settings->myaddr)
	gtk_entry_set_text(wnd->myaddr, settings->myaddr);
    else
	gtk_entry_set_text(wnd->myaddr, settings->global->myaddr ?
			   settings->global->myaddr : "");
}

void loadSecureTab(site_manager_t wnd, pftp_settings_t settings)
{
    setCheckButton(wnd->use_secure, settings->use_secure);
    setCheckButton(wnd->secure_data, settings->secure_data);
    setCheckButton(wnd->secure_list, settings->secure_list);

    if (settings->implicid_ssl == 1) {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wnd->implicid_ssl),
				     TRUE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wnd->auth_tls),
				     FALSE);
    } else if (settings->implicid_ssl == 0) {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wnd->implicid_ssl),
				     FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wnd->auth_tls),
				     TRUE);
    } else {
	gtk_toggle_button_set_inconsistent(GTK_TOGGLE_BUTTON(wnd->implicid_ssl), TRUE);
	gtk_toggle_button_set_inconsistent(GTK_TOGGLE_BUTTON(wnd->auth_tls), 
					   TRUE);
    }

    if (settings->use_secure == 1) {
	gtk_widget_set_sensitive(GTK_WIDGET(wnd->implicid_ssl), TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(wnd->auth_tls), TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(wnd->secure_data), TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(wnd->secure_list), TRUE);
    } else {
	gtk_widget_set_sensitive(GTK_WIDGET(wnd->implicid_ssl), FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(wnd->auth_tls), FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(wnd->secure_data), FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(wnd->secure_list), FALSE);
    }
}

void loadPrioTab(site_manager_t wnd, pftp_settings_t settings)
{
    GtkTreeIter iter;
    size_t m;

    if (settings->include_global_prio)
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wnd->include_global_prio), TRUE);
    else
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wnd->include_global_prio), FALSE);

    gtk_entry_set_text(wnd->prio_mask, "");
    
    gtk_list_store_clear(wnd->prio_list);

    for (m = 0; m < settings->prio_list_len; m++) {
	gtk_list_store_append(wnd->prio_list, &iter);
	gtk_list_store_set(wnd->prio_list, &iter, 0, settings->prio_list[m], 
			   -1);
    }

    gtk_tree_view_columns_autosize(wnd->priority_list);
}

void loadSkipTab(site_manager_t wnd, pftp_settings_t settings)
{
    GtkTreeIter iter;
    size_t m;

    if (settings->include_global_ignore)
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wnd->include_global_skip), TRUE);
    else
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wnd->include_global_skip), FALSE);

    gtk_entry_set_text(wnd->skip_mask, "");
    
    gtk_list_store_clear(wnd->skip_list);

    for (m = 0; m < settings->ignore_list_len; m++) {
	gtk_list_store_append(wnd->skip_list, &iter);
	gtk_list_store_set(wnd->skip_list, &iter, 0, settings->ignore_list[m], 
			   -1);
    }

    gtk_tree_view_columns_autosize(wnd->ignore_list);
}

void setCheckButton(GtkCheckButton *btn, int val)
{
    if (val == 1) {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(btn), TRUE);
    } else if (val == 0) {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(btn), FALSE);
    } else {
	gtk_toggle_button_set_inconsistent(GTK_TOGGLE_BUTTON(btn), TRUE);
    }
}

void siteManagerAddHost(void)
{
    // !!!
}

void siteManagerDelHost(void)
{
    GtkTreeSelection *sel = gtk_tree_view_get_selection(site_manager->host_list);
    GtkTreeIter iter;

    if (gtk_tree_selection_get_selected(sel, NULL, &iter)) {
	gtk_list_store_remove(site_manager->host_store, &iter);
    }
}

void siteManagerEditHost(void)
{
    // !!!
}

void siteManagerUseSecure(void)
{
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(site_manager->use_secure))) {
	gtk_widget_set_sensitive(GTK_WIDGET(site_manager->implicid_ssl), TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(site_manager->auth_tls), TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(site_manager->secure_data), TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(site_manager->secure_list), TRUE);
    } else {
	gtk_widget_set_sensitive(GTK_WIDGET(site_manager->implicid_ssl), 
				 FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(site_manager->auth_tls), FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(site_manager->secure_data), FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(site_manager->secure_list), FALSE);
    }    
}

void siteManagerPrioListSelection(void)
{
    GtkTreeSelection *sel = gtk_tree_view_get_selection(site_manager->priority_list);
    GtkTreeIter iter;

    if (gtk_tree_selection_get_selected(sel, NULL, &iter)) {
	GValue value;
	memset(&value, 0, sizeof(GValue));
	gtk_tree_model_get_value(GTK_TREE_MODEL(site_manager->prio_list), 
				 &iter, 0, &value);
	gtk_entry_set_text(site_manager->prio_mask, 
			   g_value_get_string(&value));
	g_value_unset(&value);
    } else {
	gtk_entry_set_text(site_manager->prio_mask, "");
    }
}

void siteManagerAddPrioMask(void)
{
    GtkTreeIter iter;
    const char *text = gtk_entry_get_text(site_manager->prio_mask);
    
    if (text && text[0]) {
	char *val = strdup(text);
	trim(val);
	gtk_list_store_append(site_manager->prio_list, &iter);
	gtk_list_store_set(site_manager->prio_list, &iter, 0, val, -1);
	free(val);
    }    
}

void siteManagerDelPrioMask(void)
{
    GtkTreeSelection *sel = gtk_tree_view_get_selection(site_manager->priority_list);
    GtkTreeIter iter;

    if (gtk_tree_selection_get_selected(sel, NULL, &iter)) {
	gtk_list_store_remove(site_manager->prio_list, &iter);
    }
}

void siteManagerEditPrioMask(void)
{
    GtkTreeSelection *sel = gtk_tree_view_get_selection(site_manager->priority_list);
    GtkTreeIter iter;
    const char *text = gtk_entry_get_text(site_manager->prio_mask);

    if (text && text[0] && gtk_tree_selection_get_selected(sel, NULL, &iter)) {
	char *val = strdup(text);
	trim(val);
	gtk_list_store_set(site_manager->prio_list, &iter, 0, val, -1);
	free(val);
    }    
}

/*
 * Function comes from GTimeTracker code.
 * gttr/gnotime/src/plug-edit.c to be exakt.
 */
/* ============================================================ */
#define ITER_EQ(a,b) (((a).stamp == (b).stamp) && \
                      ((a).user_data == (b).user_data) && \
                      ((a).user_data2 == (b).user_data2) && \
                      ((a).user_data3 == (b).user_data3))

static gboolean
gtk_tree_model_iter_prev (GtkTreeModel *tree_model, GtkTreeIter  *iter)
{
    int rc;
    GtkTreeIter cur, prev;
    
    rc = gtk_tree_model_get_iter_first (tree_model, &cur);
    if (ITER_EQ (cur, *iter)) return 0;
    while (rc)
    {
	prev = cur;
	rc = gtk_tree_model_iter_next (tree_model, &cur);
	if (0 == rc) break;
	if (ITER_EQ (cur, *iter))
	{
	    *iter = prev;
	    return 1;
	}
    }
    return 0;
}

void siteManagerMoveUpPrioMask(void)
{
    GtkTreeSelection *sel = gtk_tree_view_get_selection(site_manager->priority_list);
    GtkTreeIter iter;

    /* !!! Does not work for some reason !!! */
    
    if (gtk_tree_selection_get_selected(sel, NULL, &iter)) {
	GtkTreeIter target = iter;
	if (gtk_tree_model_iter_prev(GTK_TREE_MODEL(site_manager->prio_list),
				     &target)) {
	    gtk_list_store_move_before(site_manager->prio_list, &iter, 
				       &target);
	}
    }   
}

void siteManagerMoveDownPrioMask(void)
{
    GtkTreeSelection *sel = gtk_tree_view_get_selection(site_manager->priority_list);
    GtkTreeIter iter;

    if (gtk_tree_selection_get_selected(sel, NULL, &iter)) {
	GtkTreeIter target = iter;
	if (gtk_tree_model_iter_next(GTK_TREE_MODEL(site_manager->prio_list),
				     &target)) {
	    gtk_list_store_move_after(site_manager->prio_list, &iter, &target);
	}
    }
}

void siteManagerIgnoreListSelection(void)
{
    GtkTreeSelection *sel = gtk_tree_view_get_selection(site_manager->ignore_list);
    GtkTreeIter iter;

    if (gtk_tree_selection_get_selected(sel, NULL, &iter)) {
	GValue value;
	memset(&value, 0, sizeof(GValue));
	gtk_tree_model_get_value(GTK_TREE_MODEL(site_manager->skip_list), 
				 &iter, 0, &value);
	gtk_entry_set_text(site_manager->skip_mask, 
			   g_value_get_string(&value));
	g_value_unset(&value);
    } else {
	gtk_entry_set_text(site_manager->skip_mask, "");
    }
}

void siteManagerAddSkipMask(void)
{
    GtkTreeIter iter;
    const char *text = gtk_entry_get_text(site_manager->skip_mask);

    if (text && text[0]) {
	char *val = strdup(text);
	trim(val);
	gtk_list_store_append(site_manager->skip_list, &iter);
	gtk_list_store_set(site_manager->skip_list, &iter, 0, val, -1);
	free(val);
    }    
}

void siteManagerDelSkipMask(void)
{
    GtkTreeSelection *sel = gtk_tree_view_get_selection(site_manager->ignore_list);
    GtkTreeIter iter;

    if (gtk_tree_selection_get_selected(sel, NULL, &iter)) {
	gtk_list_store_remove(site_manager->skip_list, &iter);
    }
}

void siteManagerEditSkipMask(void)
{
    GtkTreeSelection *sel = gtk_tree_view_get_selection(site_manager->ignore_list);
    GtkTreeIter iter;
    const char *text = gtk_entry_get_text(site_manager->skip_mask);

    if (text && text[0] && gtk_tree_selection_get_selected(sel, NULL, &iter)) {
	char *val = strdup(text);
	trim(val);
	gtk_list_store_set(site_manager->skip_list, &iter, 0, val, -1);
	free(val);
    }    
}

void siteManagerNewSite(void)
{
    pftp_default_t def = pftp_getDefaultSettings();
    GtkTreeIter iter;
    GtkTreeSelection *sel = gtk_tree_view_get_selection(site_manager->site_tree);							

    pftp_createFTPSettings("[new]", NULL, NULL, NULL, 0, def ?
			   def->use_secure : -1, 0);

    gtk_list_store_append(site_manager->site_list, &iter);
    gtk_list_store_set(site_manager->site_list, &iter, 0, "[new]", -1);
    gtk_tree_selection_select_iter(sel, &iter);

    gtk_tree_view_columns_autosize(site_manager->site_tree);
}
