/* -*- pftp-c -*- */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif
#if HAVE_STDLIB_H
# include <stdlib.h>
#endif
#if HAVE_STRING_H
# include <string.h>
#endif

#include "utf8.h"

#ifdef DEBUG
# include <assert.h>
#else
# define assert(x) // x
#endif

#ifdef WITH_DMALLOC
# include <dmalloc.h>
#endif

void make_buffer_utf8_safe(char **buffer, size_t *buflen)
{
    register size_t pos;
    register unsigned char orig;
    size_t len;

    assert(buffer);

    if (buflen)
	len = *buflen;
    else
	len = strlen(*buffer);

    pos = 0;

    while (pos < len) {
	if (((unsigned char)(*buffer)[pos]) < 0x80) {
	    pos++;
	} else {
	    orig = (*buffer)[pos];
	    (*buffer) = realloc((*buffer), (len + 2));
	    memmove((*buffer) + pos + 2, (*buffer) + pos + 1, len - pos);
	    len++;
	    (*buffer)[pos++] = (char)(0xC0 | ((orig & 0xC0) >> 6));
	    (*buffer)[pos++] = (char)(0x80 | ((orig & 0x3F)));	    
	}
    }

    if (buflen)
	*buflen = len;
}

void make_ascii_again(char *buffer, size_t *buflen)
{
    register size_t pos;
    register unsigned char orig;
    size_t len;

    assert(buffer);

    if (buflen)
	len = *buflen;
    else
	len = strlen(buffer);

    pos = 0;

    while (pos < len) {
	if (((unsigned char)buffer[pos]) < 0x80) {
	    pos++;
	} else {
	    orig = ((unsigned char)buffer[pos + 1]) & 0x3F;
	    orig |= ((unsigned char)buffer[pos] & 0x3) << 6;
	    buffer[pos] = (char)orig;
	    pos++;
	    memmove(buffer + pos, buffer + pos + 1, len - pos);
	    len--;
	}
    }

    if (buflen)
	*buflen = len;
}
